import re
from typing import Tuple

from langchain_community.document_loaders import YoutubeLoader


class YoutubeTranscript:
    """
    This class provides functionality to retrieve the transcript and title
    of a YouTube video given its URL, and clean the text for further processing.
    """

    def download_from_url(self, video_url: str) -> Tuple[str, str]:
        """
        Download the transcript and title of a YouTube video, based on the URL.

        Args:
            video_url: The URL of the YouTube video. Example: "https://www.youtube.com/watch?v=VIDEO_ID"

        Returns:
            The title and transcript of the YouTube video.

        Raises:
            ValueError: If the URL is not valid.
            IndexError: If the video does not have a transcript.
        """
        try:
            # Try loading the transcript from the YouTube URL - not all videos have transcripts
            loader = YoutubeLoader.from_youtube_url(
                youtube_url=video_url, language="en", add_video_info=True
            )
            yt_document = loader.load()[0]
        except ValueError:
            raise ValueError(
                "Please provide a valid YouTube URL "
                'in the form of "https://www.youtube.com/watch?v=VIDEO_ID".'
            )
        except IndexError:
            raise IndexError(
                "Video does not have a transcript. Please try another video."
            )

        return (
            self._clean_text(yt_document.metadata.get("title")),
            self._clean_text(yt_document.page_content),
        )

    @staticmethod
    def _clean_text(text: str) -> str:
        """
        Clean the text of unwanted characters and whitespaces.

        Args:
            text: The text to clean.

        Returns:
            The cleaned text.
        """
        # Remove unwanted characters
        cleaned = re.sub(r"(\xa0|\n|\[Music])", " ", text)
        # Replace multiple whitespaces with single whitespace
        return re.sub(r"\s+", " ", cleaned)



import typer
from langchain_core.prompts import PromptTemplate
from langchain_core.runnables import RunnableSerializable
from langchain_openai import ChatOpenAI
from openai import AuthenticationError
from rich import print

from ltf import YoutubeTranscript
from ltf import utils
from ltf.config import settings
from ltf.models import FlashcardSet


class LanguageTransferFlashcards:
    """
    Create Flashcards from Language-Transfer YouTube videos using ChatGPT.

    This class provides functionality to download YouTube transcripts,
    process them with a language model, and generate flashcards.
    """

    def __init__(self, url: str, target_language: str):
        """
        Initialize the LanguageTransferFlashcards class

        Args:
            url: URL of the YouTube video
            target_language: The language that is taught in the YouTube video
        """
        self.title, self.transcript = YoutubeTranscript().download_from_url(url)
        self.target_language = target_language
        self.prompt_template = PromptTemplate(
            template=utils.load_template(),
            input_variables=["video_title", "target_language", "youtube_transcript"],
        )

    def _get_chain(self, llm: ChatOpenAI) -> RunnableSerializable:
        """
        Build the LLM chain to create flashcards.

        Args:
            llm: LLM model to use

        Returns:
            LLM chain
        """
        structured_output_llm = llm.with_structured_output(FlashcardSet)
        return self.prompt_template | structured_output_llm

    def _create_flashcards(self, llm: ChatOpenAI) -> FlashcardSet:
        """
        Create flashcards consisting of English and target language translations

        Args:
            llm: LLM model to use

        Returns:
            A set of flashcards

        Raises:
            AuthenticationError: If OpenAI API key is invalid
        """
        extraction_chain = self._get_chain(llm=llm)
        try:
            return extraction_chain.invoke(
                {
                    "video_title": self.title,
                    "target_language": self.target_language,
                    "youtube_transcript": self.transcript,
                }
            )
        except AuthenticationError:
            print(
                "Incorrect API key provided: "
                "You can find your API key at https://platform.openai.com/account/api-keys.\n"
                "Please update the value in your .env file."
            )
            raise typer.Abort()

    def run(self, model_name: str, api_key: str, delimiter: str, exclude: str) -> None:
        """
        Create Flashcards from YouTube video and save them as CSV file

        Args:
            model_name: OpenAI model name to use
            api_key: OpenAI API key
            delimiter: Delimiter to use in CSV file
            exclude: Directory containing CSV files with words and sentences to exclude
        """
        llm = utils.initialize_llm(
            api_key=api_key if api_key else settings.OPENAI_API_KEY,
            model_name=model_name if model_name else settings.OPENAI_MODEL_NAME,
        )
        print(f"Using: [green bold]{llm.model_name}[/green bold]\n")

        flashcards = self._create_flashcards(llm=llm)
        utils.save_flashcards_as_csv(
            flashcards, filename=self.title, delimiter=delimiter, exclude=exclude
        )

    def save_prompt(self) -> None:
        """Create final prompt and save it as text file."""
        prompt_as_string = self.prompt_template.invoke(
            {
                "video_title": self.title,
                "target_language": self.target_language,
                "youtube_transcript": self.transcript,
            }
        ).text

        utils.save_prompt_as_txt(prompt_as_string, filename=self.title)



from pathlib import Path
from typing import Optional

from pydantic_settings import BaseSettings, SettingsConfigDict

from ltf.models import AvailableTargetLanguages, AvailableModels

ENV_DIR = Path.home() / ".ltf"


class Settings(BaseSettings):
    # OPENAI settings
    OPENAI_API_KEY: Optional[str] = None
    OPENAI_MODEL_NAME: AvailableModels = AvailableModels.GPT_4o

    # The language you are currently learning form Language Transfer
    TARGET_LANGUAGE: Optional[AvailableTargetLanguages] = None

    model_config = SettingsConfigDict(env_file=(ENV_DIR / ".env"))


settings = Settings()
